﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using System.Reflection;

namespace RunAs32Bit
{
  class Program
  {

    //Quelle: http://www.lostechies.com/blogs/gabrielschenker/archive/2009/10/21/force-net-application-to-run-in-32bit-process-on-64bit-os.aspx

    [STAThread]
    static int Main(string[] args)
    {

      // load the assembly

      var directory = Path.GetDirectoryName(Assembly.GetExecutingAssembly().Location);

      var assemblyName = Path.Combine(directory, args[0].Substring(10));


      var assembly = Assembly.LoadFile(assemblyName);



      // find the entry point of the assembly
      var type = assembly.GetTypes().FirstOrDefault(t => t.GetMethod("Main") != null);
      MethodInfo mi;
      if (type == null)
      {
        //Fallback: search for method "Main" without modifier "public" (that's default on a newly created project with Visual Studio 2019)
        type = assembly.GetTypes().FirstOrDefault(t => t.GetMethod("Main", BindingFlags.NonPublic | BindingFlags.Static) != null);
        if (type == null)
        {
          Console.Error.WriteLine("Could not find type with method 'Main' in assembly " + assembly.Location);
          return -1;
        }
        else
        {
          //Search the main method also "non public":
          mi = type.GetMethod("Main", BindingFlags.NonPublic | BindingFlags.Static);
        }
      }
      else
      {
        mi = type.GetMethod("Main");
      }
      if (mi == null)
      {
        Console.Error.WriteLine($"Could not find method 'Main' for type {type.FullName}  in assembly {assembly.Location}");
        return -1;
      }


      // call the entry point of the wrapped assembly and forward the command line parameters
      //Main can have no parameters (for projects created from project templates of Visual Studio 2019)
      if (mi.GetParameters().Length == 0)
      {
        //Raise error if parameters are specified?
        object objReturn = mi.Invoke(type, null);
        if (objReturn != null && objReturn is int)
        {
          return (int)objReturn;
        }
        else
        {
          return 0;
        }
      }
      else
      {
        var arguments = args[1].Substring(8).Split(' ');

        var list = arguments.Where(a => a.Trim() != "").ToArray();

        //return (int)mi.Invoke(type, new object[] { list });
        object objReturn = mi.Invoke(type, new object[] { list });
        if (objReturn != null && objReturn is int)
        {
          return (int)objReturn;
        }
        else
        {
          return 0;
        }
      }

    }


  }
}
